--Libraries--------------------------------------------------------------------
local LAM = LibStub("LibAddonMenu-2.0")
local LMP = LibStub("LibMapPins-1.0")

--Local constants -------------------------------------------------------------
local ADDON_VERSION = "1.0.1"
local PINS_INCOMPLETE = "APBMapPin_incomplete"
local PINS_COMPLETE = "APBMapPin_complete"
local PINS_COMPASS = "APBCompassPin_incomplete"

--Local variables -------------------------------------------------------------
local updatePins = {}
local updating = false
local savedVariables
local defaults = {
   compassMaxDistance = 0.05,
   pinTexture = {
      type = 1,
      size = 38,
      level = 40,
   },
   filters = {
      [PINS_COMPASS] = true,
      [PINS_INCOMPLETE] = true,
      [PINS_COMPLETE] = true,
   },
}

-- Local functions ------------------------------------------------------------
local function MyPrint(...)
   CHAT_SYSTEM:AddMessage(...)
end

-- Pins -----------------------------------------------------------------------
local pinTexturesList = {
   [1] = "Default icons",
}

local pinTextures = {
   unknown = {
      [1] = "/esoui/art/icons/poi/poi_groupboss_incomplete.dds",
   },
   collected = {
      [1] = "/esoui/art/icons/poi/poi_groupboss_complete.dds",
   },
}

--tooltip creator
local pinTooltipCreator = {}
pinTooltipCreator.tooltip = InformationTooltip
pinTooltipCreator.creator = function(pin)
   local _, pinTag = pin:GetPinTypeAndTag()
   local name = GetAchievementInfo(pinTag[3])
   local description, numCompleted = GetAchievementCriterion(pinTag[3], pinTag[4]) 
   local info = {}

   if pinTag[5] ~= nil then
      table.insert(info, "[" .. GetString("APB_MOREINFO", pinTag[5]) .. "]")
   end
   if numCompleted == 1 then
      table.insert(info, "[" .. GetString(APB_KILLED) .. "]")
   end

   InformationTooltip:AddLine(zo_strformat("<<1>>", name), "ZoFontGameOutline", ZO_SELECTED_TEXT:UnpackRGB())
   ZO_Tooltip_AddDivider(InformationTooltip)
   InformationTooltip:AddLine(zo_strformat("(<<1>>) <<2>>", pinTag[4], description), "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
   if info[1] then
      InformationTooltip:AddLine(table.concat(info, " / "), "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
   end
end

--add callbacks
local function CreatePins()
   local zone, subzone = LMP:GetZoneAndSubzone()
   local dungeons = APB_GetLocalData(zone, subzone)

   if dungeons ~= nil then
      for _, pinData in ipairs(dungeons) do
         local _, numCompleted = GetAchievementCriterion(pinData[3], pinData[4])
         if numCompleted == 1 and updatePins[PINS_COMPLETE] and LMP:IsEnabled(PINS_COMPLETE) then
            LMP:CreatePin(PINS_COMPLETE, pinData, pinData[1], pinData[2])
         elseif numCompleted == 0 then
            if updatePins[PINS_INCOMPLETE] and LMP:IsEnabled(PINS_INCOMPLETE) then
               LMP:CreatePin(PINS_INCOMPLETE, pinData, pinData[1], pinData[2])
            end
            if updatePins[PINS_COMPASS] and savedVariables.filters[PINS_COMPASS] then
               COMPASS_PINS.pinManager:CreatePin(PINS_COMPASS, pinData, pinData[1], pinData[2])
            end
         end
      end
   end

   updatePins = {}
   updating = false
end

local function QueueCreatePins(pinType)
   updatePins[pinType] = true

   if not updating then
      updating = true
      if IsPlayerActivated() then
         zo_callLater(CreatePins, 50)
      else
         EVENT_MANAGER:RegisterForEvent("APB_PinUpdate", EVENT_PLAYER_ACTIVATED,
            function(event)
               EVENT_MANAGER:UnregisterForEvent("APB_PinUpdate", event)
               CreatePins()
            end)
      end
   end
end

local function MapCallback_incomplete()
   if not LMP:IsEnabled(PINS_INCOMPLETE) or (GetMapType() > MAPTYPE_ZONE) then return end
   QueueCreatePins(PINS_INCOMPLETE)
end

local function MapCallback_complete()
   if not LMP:IsEnabled(PINS_COMPLETE) or (GetMapType() > MAPTYPE_ZONE) then return end
   QueueCreatePins(PINS_COMPLETE)
end

local function CompassCallback()
   if not savedVariables.filters[PINS_COMPASS] or (GetMapType() > MAPTYPE_ZONE) then return end
   QueueCreatePins(PINS_COMPASS)
end

-- Slash commands -------------------------------------------------------------
local function ShowMyPosition()
   local x, y = GetMapPlayerPosition("player")

   local locX = ("%05.02f"):format(zo_round(x*10000)/100)
   local locY = ("%05.02f"):format(zo_round(y*10000)/100)

   MyPrint(zo_strformat("<<1>>: <<2>>\195\151<<3>> (<<4>>)", GetMapName(), locX, locY, LMP:GetZoneAndSubzone(true)))
end
SLASH_COMMANDS["/mypos"] = ShowMyPosition
SLASH_COMMANDS["/myposition"] = ShowMyPosition
SLASH_COMMANDS["/myloc"] = ShowMyPosition
SLASH_COMMANDS["/mylocation"] = ShowMyPosition

-- Settings menu --------------------------------------------------------------
local function CreateSettingsMenu()
   local panelData = {
      type = "panel",
      name = GetString(APB_TITLE),
      displayName = ZO_HIGHLIGHT_TEXT:Colorize(GetString(APB_TITLE)),
      author = "Joe_Schmo2840",
      version = ADDON_VERSION,
      slashCommand = "/apb",
      registerForRefresh = true,
      registerForDefaults = true,
   }
   LAM:RegisterAddonPanel("APB", panelData)

   local CreateIcons, incompleteIcon, completeIcon
   CreateIcons = function(panel)
      if panel == APB then
         incompleteIcon = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[1], CT_TEXTURE)
         incompleteIcon:SetAnchor(RIGHT, panel.controlsToRefresh[1].dropdown:GetControl(), LEFT, -10, 0)
         incompleteIcon:SetTexture(pinTextures.unknown[savedVariables.pinTexture.type])
         incompleteIcon:SetDimensions(savedVariables.pinTexture.size, savedVariables.pinTexture.size)
         completeIcon = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[1], CT_TEXTURE)
         completeIcon:SetAnchor(RIGHT, incompleteIcon, LEFT, -5, 0)
         completeIcon:SetTexture(pinTextures.collected[savedVariables.pinTexture.type])
         completeIcon:SetDimensions(savedVariables.pinTexture.size, savedVariables.pinTexture.size)
         CALLBACK_MANAGER:UnregisterCallback("LAM-PanelControlsCreated", CreateIcons)
      end
   end
   CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", CreateIcons)

   local optionsTable = {
      {
         type = "dropdown",
         name = GetString(APB_PIN_TEXTURE),
         tooltip = GetString(APB_PIN_TEXTURE_DESC),
         choices = pinTexturesList,
         getFunc = function() return pinTexturesList[savedVariables.pinTexture.type] end,
         setFunc = function(selected)
               for index, name in ipairs(pinTexturesList) do
                  if name == selected then
                     savedVariables.pinTexture.type = index
                     LMP:SetLayoutKey(PINS_INCOMPLETE, "texture", pinTextures.unknown[index])
                     LMP:SetLayoutKey(PINS_COMPLETE, "texture", pinTextures.collected[index])
                     incompleteIcon:SetTexture(pinTextures.unknown[index])
                     completeIcon:SetTexture(pinTextures.collected[index])
                     LMP:RefreshPins(PINS_INCOMPLETE)
                     LMP:RefreshPins(PINS_COMPLETE)
                     COMPASS_PINS.pinLayouts[PINS_COMPASS].texture = pinTextures.unknown[index]
                     COMPASS_PINS:RefreshPins(PINS_COMPASS)
                     break
                  end
               end
            end,
         disabled = function() return not (savedVariables.filters[PINS_INCOMPLETE] or savedVariables.filters[PINS_COMPLETE]) end,
         default = pinTexturesList[defaults.pinTexture.type],
      },
      {
         type = "slider",
         name = GetString(APB_PIN_SIZE),
         tooltip = GetString(APB_PIN_SIZE_DESC),
         min = 20,
         max = 70,
         getFunc = function() return savedVariables.pinTexture.size end,
         setFunc = function(size)
               savedVariables.pinTexture.size = size
               incompleteIcon:SetDimensions(size, size)
               completeIcon:SetDimensions(size, size)
               LMP:SetLayoutKey(PINS_INCOMPLETE, "size", size)
               LMP:SetLayoutKey(PINS_COMPLETE, "size", size)
               LMP:RefreshPins(PINS_INCOMPLETE)
               LMP:RefreshPins(PINS_COMPLETE)
            end,
         disabled = function() return not (savedVariables.filters[PINS_INCOMPLETE] or savedVariables.filters[PINS_COMPLETE]) end,
         default = defaults.pinTexture.size
      },
      {
         type = "slider",
         name = GetString(APB_PIN_LAYER),
         tooltip = GetString(APB_PIN_LAYER_DESC),
         min = 10,
         max = 200,
         step = 5,
         getFunc = function() return savedVariables.pinTexture.level end,
         setFunc = function(level)
               savedVariables.pinTexture.level = level
               LMP:SetLayoutKey(PINS_INCOMPLETE, "level", level)
               LMP:SetLayoutKey(PINS_COMPLETE, "level", level)
               LMP:RefreshPins(PINS_INCOMPLETE)
               LMP:RefreshPins(PINS_COMPLETE)
            end,
         disabled = function() return not (savedVariables.filters[PINS_INCOMPLETE] or savedVariables.filters[PINS_COMPLETE]) end,
         default = defaults.pinTexture.level,
      },
      {
         type = "checkbox",
         name = GetString(APB_INCOMPLETE),
         tooltip = GetString(APB_INCOMPLETE_DESC),
         getFunc = function() return savedVariables.filters[PINS_INCOMPLETE] end,
         setFunc = function(state)
               savedVariables.filters[PINS_INCOMPLETE] = state
               LMP:SetEnabled(PINS_INCOMPLETE, state)
            end,
         default = defaults.filters[PINS_INCOMPLETE],
      },
      {
         type = "checkbox",
         name = GetString(APB_COMPLETE),
         tooltip = GetString(APB_COMPLETE_DESC),
         getFunc = function() return savedVariables.filters[PINS_COMPLETE] end,
         setFunc = function(state)
               savedVariables.filters[PINS_COMPLETE] = state
               LMP:SetEnabled(PINS_COMPLETE, state)
            end,
         default = defaults.filters[PINS_COMPLETE]
      },
      {
         type = "checkbox",
         name = GetString(APB_COMPASS_INCOMPLETE),
         tooltip = GetString(APB_COMPASS_INCOMPLETE_DESC),
         getFunc = function() return savedVariables.filters[PINS_COMPASS] end,
         setFunc = function(state)
               savedVariables.filters[PINS_COMPASS] = state
               COMPASS_PINS:RefreshPins(PINS_COMPASS)
            end,
         default = defaults.filters[PINS_COMPASS],
      },
      {
         type = "slider",
         name = GetString(APB_COMPASS_DIST),
         tooltip = GetString(APB_COMPASS_DIST_DESC),
         min = 1,
         max = 100,
         getFunc = function() return savedVariables.compassMaxDistance * 1000 end,
         setFunc = function(maxDistance)
               savedVariables.compassMaxDistance = maxDistance / 1000
               COMPASS_PINS.pinLayouts[PINS_COMPASS].maxDistance = maxDistance / 1000
               COMPASS_PINS:RefreshPins(PINS_COMPASS)
            end,
         width = "full",
         disabled = function() return not savedVariables.filters[PINS_COMPASS] end,
         default = defaults.compassMaxDistance * 1000,
      },
   }
   LAM:RegisterOptionControls("APB", optionsTable)
end

-- Event handlers -------------------------------------------------------------
local function OnAchievementUpdate(eventCode, achievementId)
   local ids = APB_GetAchievementIDs()

   for _, id in ipairs(ids) do
      if id == achievementId then
         LMP:RefreshPins(PINS_INCOMPLETE)
         LMP:RefreshPins(PINS_COMPLETE)
         COMPASS_PINS:RefreshPins(PINS_COMPASS)
         break
      end
   end
end

local function OnLoad(eventCode, name)
   if name ~= "APB" then return end

   savedVariables = ZO_SavedVars:New("APB_SavedVariables", 4, nil, defaults)

   --get pin layout from saved variables
   local pinTextureType = savedVariables.pinTexture.type
   local pinTextureLevel = savedVariables.pinTexture.level
   local pinTextureSize = savedVariables.pinTexture.size
   local compassMaxDistance = savedVariables.compassMaxDistance

   local pinLayout_incomplete = { level = pinTextureLevel, texture = pinTextures.unknown[pinTextureType], size = pinTextureSize }
   local pinLayout_complete = { level = pinTextureLevel, texture = pinTextures.collected[pinTextureType], size = pinTextureSize }
   local pinLayout_compass = {
      maxDistance = compassMaxDistance,
      texture = pinTextures.unknown[pinTextureType],
      sizeCallback = function(pin, angle, normalizedAngle, normalizedDistance)
         if zo_abs(normalizedAngle) > 0.25 then
            pin:SetDimensions(54 - 24 * zo_abs(normalizedAngle), 54 - 24 * zo_abs(normalizedAngle))
         else
            pin:SetDimensions(48, 48)
         end
      end
   }

   --initialize map pins
   LMP:AddPinType(PINS_INCOMPLETE, MapCallback_incomplete, nil, pinLayout_incomplete, pinTooltipCreator)
   LMP:AddPinType(PINS_COMPLETE, MapCallback_complete, nil, pinLayout_complete, pinTooltipCreator)

   --add filter check boxex
   LMP:AddPinFilter(PINS_INCOMPLETE, GetString(APB_FILTER_INCOMPLETE), nil, savedVariables.filters)
   LMP:AddPinFilter(PINS_COMPLETE, GetString(APB_FILTER_COMPLETE), nil, savedVariables.filters)

   --initialize compass pins
   COMPASS_PINS:AddCustomPin(PINS_COMPASS, CompassCallback, pinLayout_compass)
   COMPASS_PINS:RefreshPins(PINS_COMPASS)

   --if ZrMM exists, register map pins
   if ZrMM_RegisterCustomPin then
      ZrMM_RegisterCustomPin(_G[PINS_INCOMPLETE], GetString(APB_FILTER_INCOMPLETE))
      ZrMM_RegisterCustomPin(_G[PINS_COMPLETE], GetString(APB_FILTER_COMPLETE))
   end

   -- addon menu
   CreateSettingsMenu()

   --events
   EVENT_MANAGER:RegisterForEvent("APB",  EVENT_ACHIEVEMENT_UPDATED, OnAchievementUpdate)

   EVENT_MANAGER:UnregisterForEvent("APB", EVENT_ADD_ON_LOADED)
end

EVENT_MANAGER:RegisterForEvent("APB", EVENT_ADD_ON_LOADED, OnLoad)
